package ru.ulstu.extractor.markup.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.core.Route;
import ru.ulstu.extractor.markup.model.MarkupForm;
import ru.ulstu.extractor.markup.model.TimeSeriesForMarkup;
import ru.ulstu.extractor.markup.service.MarkupService;
import ru.ulstu.extractor.ts.service.TimeSeriesService;
import springfox.documentation.annotations.ApiIgnore;

import java.util.List;

@Controller
@ApiIgnore
public class TimeSeriesMarkupController {
    private final TimeSeriesService timeSeriesService;
    private final BranchService branchService;
    private final MarkupService markupService;

    public TimeSeriesMarkupController(TimeSeriesService timeSeriesService,
                                      BranchService branchService,
                                      MarkupService markupService) {
        this.timeSeriesService = timeSeriesService;
        this.branchService = branchService;
        this.markupService = markupService;
    }

    @GetMapping("time-series-markup")
    public String markupTs(Model model) {
        model.addAttribute("branches", branchService.findAllValid());
        model.addAttribute("markupForm", new MarkupForm());
        return "markup";
    }

    @PostMapping("time-series-markup")
    public String filter(Model model, @ModelAttribute MarkupForm markupForm) {
        model.addAttribute("branches", branchService.findAllValid());
        if (markupForm != null && markupForm.getBranchId() != null) {
            List<TimeSeriesForMarkup> tss = markupService.getTimeSeriesForMarkup(
                    timeSeriesService.getGroupedTendencies(
                            timeSeriesService.getByBranch(markupForm.getBranchId())
                    )
            );
            MarkupForm markupFormWithTs = new MarkupForm(markupForm.getBranchId(), tss);
            model.addAttribute("markupForm", markupFormWithTs);
        } else {
            model.addAttribute("markupForm", new MarkupForm());
        }
        return "markup";
    }

    @PostMapping(Route.ADD_MARKUP)
    public String addMarkups(Model model, @ModelAttribute MarkupForm markupForm) {
        model.addAttribute("branches", branchService.findAllValid());
        if (markupForm != null && markupForm.getBranchId() != null) {
            List<TimeSeriesForMarkup> tss = markupService.getTimeSeriesForMarkup(
                    timeSeriesService.getGroupedTendencies(
                            timeSeriesService.getByBranch(markupForm.getBranchId())
                    )
            );
            MarkupForm markupFormWithTs = new MarkupForm(markupForm.getBranchId(), tss);
            model.addAttribute("markupForm", markupFormWithTs);
//            if (markupForm.getMarkupRows() != null && !markupForm.getMarkupRows().isEmpty()) {
//                markupService.saveMarkup(markupForm);
//            }
        }
        return "markup";
    }
}
