package ru.ulstu.extractor.rule.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.extractor.markup.model.Markup;
import ru.ulstu.extractor.markup.service.MarkupService;
import ru.ulstu.extractor.rule.model.AddRuleForm;
import ru.ulstu.extractor.rule.model.AntecedentValue;
import ru.ulstu.extractor.rule.model.DbRule;
import ru.ulstu.extractor.rule.repository.RuleRepository;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class DbRuleService {
    private final RuleRepository ruleRepository;
    private final AntecedentValueService antecedentValueService;
    private final MarkupService markupService;

    public DbRuleService(RuleRepository ruleRepository,
                         TimeSeriesService timeSeriesService,
                         AntecedentValueService antecedentValueService,
                         MarkupService markupService) {
        this.ruleRepository = ruleRepository;
        this.antecedentValueService = antecedentValueService;
        this.markupService = markupService;
    }

    public List<DbRule> getList() {
        return ruleRepository.findAll();
    }

    public void saveRule(AddRuleForm addRuleForm) {
        if (addRuleForm.getRuleId() != null) {
            DbRule rule = ruleRepository.getOne(addRuleForm.getRuleId());
            rule.setConsequent(addRuleForm.getConsequent());
            rule.setFirstAntecedent(TimeSeriesType.valueOf(addRuleForm.getFirstAntecedentId()));
            rule.setFirstAntecedentValue(antecedentValueService.getById(addRuleForm.getFirstAntecedentValueId()));
            rule.setSecondAntecedent(TimeSeriesType.valueOf(addRuleForm.getSecondAntecedentId()));
            rule.setSecondAntecedentValue(antecedentValueService.getById(addRuleForm.getSecondAntecedentValueId()));
            ruleRepository.save(rule);
        } else {
            ruleRepository.save(new DbRule(antecedentValueService.getById(addRuleForm.getFirstAntecedentValueId()),
                    TimeSeriesType.valueOf(addRuleForm.getFirstAntecedentId()),
                    antecedentValueService.getById(addRuleForm.getSecondAntecedentValueId()),
                    TimeSeriesType.valueOf(addRuleForm.getSecondAntecedentId()),
                    addRuleForm.getConsequent()));
        }
    }

    public DbRule findById(Integer id) {
        return ruleRepository.getOne(id);
    }

    public AddRuleForm getAddRuleFormOrDefault(Integer id) {
        return id != null
                ? new AddRuleForm(ruleRepository
                .findById(id)
                .orElseThrow(() -> new RuntimeException("Правило не найдено")))
                : new AddRuleForm();
    }

    public void deleteById(Integer id) {
        ruleRepository.deleteById(id);
    }

    public List<String> getConsequentList() {
        return ruleRepository.findAll().stream().map(DbRule::getConsequent).collect(Collectors.toList());
    }

    @Transactional
    public void generateRulesFromMarkup() {
        List<Markup> markups = markupService.findAll();
        for (Markup markup1 : markups) {
            for (Markup markup2 : markups) {
                if (!markup1.equals(markup2)) {
                    List<TimeSeriesValue> values1 = markup1.getTimeSeries().getValues();
                    for (int i = 1; i < values1.size(); i++) {
                        List<TimeSeriesValue> values2 = markup2.getTimeSeries().getValues();
                        for (int j = 1; j < values2.size(); j++) {
                            ruleRepository.save(new DbRule(
                                    getAntecedent(values1.get(i).getValue() - values1.get(i - 1).getValue()),
                                    markup1.getTimeSeries().getTimeSeriesType(),
                                    getAntecedent(values2.get(j).getValue() - values2.get(j - 1).getValue()),
                                    markup2.getTimeSeries().getTimeSeriesType(),
                                    "сгенерированный консеквент"
                            ));
                        }
                    }
                }
            }
        }
    }

    private AntecedentValue getAntecedent(Double diff) {
        String antecedentValue;
        if (diff < 0) {
            antecedentValue = "спад";
        } else if (diff > 0) {
            antecedentValue = "рост";
        } else {
            antecedentValue = "стабильно";
        }
        return antecedentValueService.getByValue(antecedentValue);
    }
}
