package ru.ulstu.extractor.ts.creator.db;

import org.springframework.stereotype.Component;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.commit.service.CommitService;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class CommitsTS extends DBTimeSeriesCreator {
    private final TimeSeriesService timeSeriesService;
    private final CommitService commitService;
    private final BranchService branchService;

    public CommitsTS(TimeSeriesService timeSeriesService,
                     CommitService commitService,
                     BranchService branchService) {
        this.timeSeriesService = timeSeriesService;
        this.commitService = commitService;
        this.branchService = branchService;
    }

    @Override
    public List<TimeSeries> getTimeSeries(Branch branch) {
        //TODO: добавить постраничное чтение
        return Collections.singletonList(
                new TimeSeries(
                        String.format("%s %s %s",
                                branch.getGitRepository().getName(),
                                branch.getName(),
                                getTimeSeriesType().getDescription()),
                        branchService.findByRepositoryAndName(branch.getGitRepository(),
                                branch.getName()),
                        getTimeSeriesType(),
                        commitService.findByRepositoryIdAndName(branch.getGitRepository().getId(),
                                        branch.getName())
                                .stream()
                                .map(c -> new TimeSeriesValue(c.getDate(), 1.0))
                                .collect(Collectors.toList())));
    }

    @Override
    public TimeSeriesService getTimeSeriesService() {
        return timeSeriesService;
    }

    @Override
    public TimeSeriesType getTimeSeriesType() {
        return TimeSeriesType.COMMITS;
    }
}
