package ru.ulstu.extractor.ts.model;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
import org.json.JSONObject;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.core.BaseEntity;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static ru.ulstu.extractor.util.JsonUtils.getListOfObjects;

@Entity
public class TimeSeries extends BaseEntity {
    private String name;
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinColumn(name = "time_series_id")
    @Fetch(FetchMode.SUBSELECT)
    private List<TimeSeriesValue> values = new ArrayList<>();

    @Enumerated(EnumType.STRING)
    private TimeSeriesType timeSeriesType;

    @ManyToOne
    private Branch branch;

    public TimeSeries() {
    }

    public TimeSeries(Branch branch, TimeSeriesType timeSeriesType) {
        this(String.format("%s %s %s",
                branch.getGitRepository().getName(),
                branch.getName(),
                timeSeriesType.getDescription()), branch, timeSeriesType);
    }

    public TimeSeries(String name, Branch branch, TimeSeriesType timeSeriesType) {
        this(name, branch, timeSeriesType, new ArrayList<>());
    }

    public TimeSeries(String name, Branch branch, TimeSeriesType timeSeriesType, List<TimeSeriesValue> values) {
        this.name = name;
        this.values = values;
        this.timeSeriesType = timeSeriesType;
        this.branch = branch;
    }

    public TimeSeries(JSONObject timeSeries) {
        this.name = timeSeries.getString("name");
        this.values = getListOfObjects(timeSeries.getJSONArray("values"))
                .stream()
                .map(TimeSeriesValue::new)
                .collect(Collectors.toList());
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<TimeSeriesValue> getValues() {
        return values;
    }

    public void setValues(List<TimeSeriesValue> values) {
        this.values = values;
    }

    public TimeSeriesType getTimeSeriesType() {
        return timeSeriesType;
    }

    public void setTimeSeriesType(TimeSeriesType timeSeriesType) {
        this.timeSeriesType = timeSeriesType;
    }

    public Branch getBranch() {
        return branch;
    }

    public void setBranch(Branch branch) {
        this.branch = branch;
    }
}
